#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

out vec4 FragColor;

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

in vec2 uvs;

uniform float nearZ = 0.1;
uniform float farZ = 100;

uniform sampler2D texturedView;
uniform sampler2D normalsView;
uniform sampler2D depthView;

mat3 vertKernel = mat3(
	-1.0, 0.0, 1.0,
	-2.0, 0.0, 2.0,
	-1.0, 0.0, 1.0
);

mat3 horzKernel = mat3(
	-1.0, -2.0, -1.0,
	0.0, 0.0, 0.0,
	1.0, 2.0, 1.0
);

// https://www.geeks3d.com/20091216/geexlab-how-to-visualize-the-depth-buffer-in-glsl/
float linearizedDepth(float depthValue)
{
	// 0.2 / 100.1 - x * 99.9
	return (2.0 * nearZ) / (farZ + nearZ - depthValue * (farZ - nearZ));	
}

vec3 sobelFilter(sampler2D textureSampler, float threshold) {

	mat3 loadedSample;
	for(int x = 0; x < 3; x++) {
		for (int y = 0; y < 3; y++) {
			vec4 sampleOfTexture = 
				texelFetch(textureSampler, ivec2(gl_FragCoord) + ivec2(x-1, y-1), 0);
			loadedSample[x][y] = length(sampleOfTexture.rgb); // length because we want the intensity for edgedetection
		}
	}
	
	float gx = dot(horzKernel[0], loadedSample[0]) 
				+ dot(horzKernel[1], loadedSample[1]) 
				+ dot(horzKernel[2], loadedSample[2]); 
	float gy = dot(vertKernel[0], loadedSample[0]) 
				+ dot(vertKernel[1], loadedSample[1]) 
				+ dot(vertKernel[2], loadedSample[2]);

	
	float g = sqrt(pow(gx, 2.0)+pow(gy, 2.0));
	if(g > threshold) {
		return vec3(g);
	} else {
		return vec3(0);
	}
}

vec3 sobelDepthFilter(float threshold) {
	
	float zFilter = 0.65;
	
	mat3 loadedSample;
	for(int x = 0; x < 3; x++) {
		for (int y = 0; y < 3; y++) {
			float depthValue = 
				texelFetch(depthView, ivec2(gl_FragCoord) + ivec2(x-1, y-1), 0).x;
			depthValue = linearizedDepth(depthValue);
			if(depthValue < zFilter) {
				loadedSample[x][y] = depthValue;  
			} else {
				loadedSample[x][y] = zFilter;  
			}
		}
	}
	
	float gx = dot(horzKernel[0], loadedSample[0]) 
				+ dot(horzKernel[1], loadedSample[1]) 
				+ dot(horzKernel[2], loadedSample[2]); 
	float gy = dot(vertKernel[0], loadedSample[0]) 
				+ dot(vertKernel[1], loadedSample[1]) 
				+ dot(vertKernel[2], loadedSample[2]);

	
	float g = sqrt(pow(gx, 2.0)+pow(gy, 2.0));
	if(g > threshold) {
		return vec3(g);
	} else {
		return vec3(0);
	}
}


vec3 sobelFilterNormal(sampler2D normalsTextureSampler, float threshold) {

	vec3 loadedSample[3][3];
	for(int x = 0; x < 3; x++) {
		for (int y = 0; y < 3; y++) {
			vec4 sampleOfTexture = 
				texelFetch(normalsTextureSampler, ivec2(gl_FragCoord) + ivec2(x-1, y-1), 0);
			loadedSample[x][y] = sampleOfTexture.rgb; // length because we want the intensity for edgedetection
		}
	}
	
	vec3 gx = vec3(0);
	for(int x = 0; x < 3; x++) {
		for (int y = 0; y < 3; y++) {
			gx.r += horzKernel[x][y] *loadedSample[x][y].r;
			gx.g += horzKernel[x][y] *loadedSample[x][y].g;
			gx.b += horzKernel[x][y] *loadedSample[x][y].b;	
		}
	}
	
	vec3 gy = vec3(0);
	for(int x = 0; x < 3; x++) {
		for (int y = 0; y < 3; y++) {
			gy.r += vertKernel[x][y] *loadedSample[x][y].r;
			gy.g += vertKernel[x][y] *loadedSample[x][y].g;
			gy.b += vertKernel[x][y] *loadedSample[x][y].b;	
		}
	}

	// gx = normalize(gx);
	// gy = normalize(gy);

	float fgx = 0;
	float fgy = 0;
	// if(gx != vec3(0)) {
		// fgx = gx.r
			// + gx.g
			// + gx.b;
	// }
	// if(gy != vec3(0)) {
		// fgx = gy.r
			// + gy.g
			// + gy.b;
	// }
	
	if(gx != vec3(0)) {
		fgx = length(gx);
	}
	if(gy != vec3(0)) {
		fgx = length(gy);
	}
	

	
	float g = sqrt(pow(fgx, 2.0) + pow(fgy, 2.0));
	if(g > threshold) {
		return vec3(g);
	} else {
		return vec3(0);
	}

}



void main() {	
    vec3 textured = texture(texturedView , vert.uv).rgb;
	vec3 depth = texture(depthView, vert.uv).rgb;
	vec3 normals = texture(normalsView, vert.uv).rgb;
	vec3 edgeColor = vec3(0.0, 0.0, 0.0);
	// vec3 col = vec3(linearizedDepth(length(depth)));
	vec3 col = textured;
	col = mix(col, edgeColor, max(sobelFilterNormal(normalsView, 0.85), sobelDepthFilter(0.2)));
    FragColor = vec4(col, 1.0);
}

